﻿Imports System.Collections
Imports System.Diagnostics

Namespace System.Drawing
   ''' <summary>a compatibily-Structure for SizeF and PointF</summary>
   <Serializable()> _
   <DebuggerDisplay("{X} {Y}")> _
   Public Structure VectorF : Implements IEnumerable(Of Single)

      Public Shared ReadOnly ControlDefaultSize As New VectorF(50, 20)
      Public Shared ReadOnly ControlMinSize As New VectorF(20, 13)

      ' both SizeF and PointF can convert to and from VectorF.
      ' X,Y also can be accessed by Index 0 or 1 as well as by Index True or False. This avoids Code-
      '°Repeating, because you don't need to implement individual computation-code for horizontal and
      '°vertical purposes - you can use the **same** code, just passing another Index.

      Public X, Y As Single
      Public Sub New(ByVal x As Single, ByVal y As Single)
         Me.X = x : Me.Y = y
      End Sub
      Public Sub New(ByVal eachMember As Single)
         X = eachMember : Y = X
      End Sub
      Public Shared Function From(ByVal eachMember As Single) As VectorF
         Return New VectorF(eachMember)
      End Function
      Public Shared Function From(ByVal x As Single, ByVal y As Single) As VectorF
         Return New VectorF(x, y)
      End Function
      Public Shared Function From(ByVal v As VectorF) As VectorF
         Return v 'implicit conversion of PointF and SizeF
      End Function
      Public Function Add(ByVal x As Single, ByVal y As Single) As VectorF
         Return New VectorF(Me.X + x, Me.Y + y)
      End Function
      Public Function Add(ByVal amount As Single) As VectorF
         Return New VectorF(X + amount, Y + amount)
      End Function
      Public Function Add(ByVal v As VectorF) As VectorF
         Return New VectorF(X + v.X, Y + v.Y)
      End Function
      Public Function Subtract(ByVal x As Single, ByVal y As Single) As VectorF
         Return New VectorF(Me.X - x, Me.Y - y)
      End Function
      Public Function Subtract(ByVal amount As Single) As VectorF
         Return New VectorF(X - amount, Y - amount)
      End Function
      Public Function Subtract(ByVal v As VectorF) As VectorF
         Return New VectorF(X - v.X, Y - v.Y)
      End Function
      Default Public Property Item(ByVal vertical As Integer) As Single
         Get
            If vertical = 0 Then Return X
            Return Y
         End Get
         Set(ByVal value As Single)
            If vertical = 0 Then X = value Else Y = value
         End Set
      End Property
      Default Public Property Item(ByVal vertical As Boolean) As Single
         Get
            If vertical Then Return Y
            Return X
         End Get
         Set(ByVal value As Single)
            If vertical Then Y = value Else X = value
         End Set
      End Property
      ''' <summary>
      ''' return-values: -1: no difference, 0: horizontal diff, 1: vertical diff, 2: diff in both directions
      ''' </summary>
      Public Function DifferenceDirection(ByVal v2 As VectorF) As Single
         Dim ret = 0
         For vertical As Integer = 0 To 1
            If Item(vertical) <> v2(vertical) Then ret = ret Or vertical + 1
         Next
         Return ret - 1
      End Function
      Public Overrides Function ToString() As String
         Return String.Concat("{", X, " ", Y, "}")
      End Function
      Public Function Scale(ByVal x As Double, ByVal y As Double) As VectorF
         Return New VectorF(CInt(Me.X * x), CInt(Me.Y * y))
      End Function
      Public Function Scale(ByVal value As Double) As VectorF
         Return New VectorF(CInt(Me.X * value), CInt(Me.Y * value))
      End Function

      Public Function Union(ByVal other As VectorF) As VectorF
         Return New VectorF(Math.Max(X, other.X), Math.Max(Y, other.Y))
      End Function
      Public Function Intersect(ByVal other As VectorF) As VectorF
         Return New VectorF(Math.Min(X, other.X), Math.Min(Y, other.Y))
      End Function
      Public Shared Widening Operator CType(ByVal sz As SizeF) As VectorF
         Return New VectorF(sz.Width, sz.Height)
      End Operator
      Public Shared Widening Operator CType(ByVal pt As PointF) As VectorF
         Return New VectorF(pt.X, pt.Y)
      End Operator
      Public Shared Widening Operator CType(ByVal v As VectorF) As PointF
         Return New PointF(v.X, v.Y)
      End Operator
      Public Shared Widening Operator CType(ByVal v As VectorF) As SizeF
         Return New SizeF(v.X, v.Y)
      End Operator
      Public Shared Operator +(ByVal v1 As SizeF, ByVal v2 As VectorF) As VectorF
         Return New VectorF(v1.Width + v2.X, v1.Height + v2.Y)
      End Operator
      Public Shared Operator +(ByVal v1 As VectorF, ByVal v2 As VectorF) As VectorF
         Return New VectorF(v1.X + v2.X, v1.Y + v2.Y)
      End Operator
      Public Shared Operator +(ByVal v1 As VectorF, ByVal sz As SizeF) As VectorF
         Return New VectorF(v1.X + sz.Width, v1.Y + sz.Height)
      End Operator
      Public Shared Operator -(ByVal v1 As SizeF, ByVal v2 As VectorF) As VectorF
         Return New VectorF(v1.Width - v2.X, v1.Height - v2.Y)
      End Operator
      Public Shared Operator -(ByVal v1 As VectorF, ByVal v2 As VectorF) As VectorF
         Return New VectorF(v1.X - v2.X, v1.Y - v2.Y)
      End Operator
      Public Shared Operator -(ByVal v1 As VectorF) As VectorF
         Return New VectorF(-v1.X, -v1.Y)
      End Operator
      Public Shared Operator -(ByVal v1 As VectorF, ByVal sz As SizeF) As VectorF
         Return New VectorF(v1.X - sz.Width, v1.Y - sz.Height)
      End Operator
      Public Shared Operator =(ByVal v1 As SizeF, ByVal v2 As VectorF) As Boolean
         Return v1.Width = v2.X AndAlso v1.Height = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As SizeF, ByVal v2 As VectorF) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As VectorF, ByVal v2 As SizeF) As Boolean
         Return v1.X = v2.Width AndAlso v1.Y = v2.Height
      End Operator
      Public Shared Operator <>(ByVal v1 As VectorF, ByVal v2 As SizeF) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As PointF, ByVal v2 As VectorF) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As PointF, ByVal v2 As VectorF) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As VectorF, ByVal v2 As PointF) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As VectorF, ByVal v2 As PointF) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As VectorF, ByVal v2 As VectorF) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As VectorF, ByVal v2 As VectorF) As Boolean
         Return Not v1 = v2
      End Operator
      Public Overrides Function Equals(ByVal obj As Object) As Boolean
         If TypeOf obj Is VectorF OrElse TypeOf obj Is PointF OrElse TypeOf obj Is SizeF Then Return Me = CType(obj, VectorF)
      End Function
      Public Overrides Function GetHashCode() As Integer
         Return X.GetHashCode Xor Y.GetHashCode
      End Function
      Public Function EachEquals(ByVal amount As Single) As Boolean
         Return X = amount AndAlso Y = amount
      End Function
      ''' <summary>enables Linq-power to Vectors Elements</summary>
      Public Function GetEnumerator() As IEnumerator(Of Single) Implements _
            IEnumerable(Of Single).GetEnumerator
         Return DirectCast(New Single() {X, Y}, IEnumerable(Of Single)) _
            .GetEnumerator
      End Function

      Private Function GetEnumerator1() As IEnumerator Implements _
            IEnumerable.GetEnumerator
         Return DirectCast(New Single() {X, Y}, IEnumerable(Of Single)) _
            .GetEnumerator
      End Function
   End Structure

End Namespace
