﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Diagnostics
Imports System.Collections

Namespace System
   <Serializable()> _
   <DebuggerDisplay("{me.Start} {End}")> _
   Public Structure FloatRange
      Implements IEnumerable(Of Single)

      Public Start As Single, [End] As Single

      'comparer support Sorting and binary Search
      Public Shared ReadOnly StartComparer As IComparer(Of FloatRange) = (New Comparison(Of FloatRange)(Function(r1, r2) r1.Start.CompareTo(r2.Start))).ToComparer()
      Public Shared ReadOnly EndComparer As IComparer(Of FloatRange) = (New Comparison(Of FloatRange)(Function(r1, r2) r1.End.CompareTo(r2.End))).ToComparer()

      Public Sub New(ByVal Start As Single, ByVal [end] As Single)
         Me.Start = Start
         Me.End = [end]
      End Sub
      Public Sub New(ByVal [each] As Single)
         Me.Start = [each]
         Me.End = [each]
      End Sub
      Public Shared Function From(ByVal start As Single) As FloatRange
         Return New FloatRange(start, start)
      End Function
      Public Shared Function From(ByVal start As Single, ByVal [end] As Single) As FloatRange
         Return New FloatRange(start, [end])
      End Function
      Public Shared Function FromLength(ByVal start As Single, ByVal length As Single) As FloatRange
         Return New FloatRange(start, start + length)
      End Function
      Public Function Add(ByVal [each] As Single) As FloatRange
         Return New FloatRange(Me.Start + [each], Me.End + [each])
      End Function
      Public Function Add(ByVal Start As Single, ByVal [end] As Single) As FloatRange
         Return New FloatRange(Me.Start + Start, Me.End + [end])
      End Function
      Public Function Subtract(ByVal [each] As Single) As FloatRange
         Return New FloatRange(Me.Start - [each], Me.End - [each])
      End Function
      Public Function Subtract(ByVal Start As Single, ByVal [end] As Single) As FloatRange
         Return New FloatRange(Me.Start - Start, Me.End - [end])
      End Function
      Default Public Property Item(ByVal end__1 As Single) As Single
         Get
            Return If(end__1 = 0, Me.Start, Me.End)
         End Get
         Set(ByVal value As Single)
            If end__1 = 0 Then
               Me.Start = value
            Else
               Me.End = value
            End If
         End Set
      End Property
      Public Overrides Function ToString() As String
         Return String.Concat("{", Me.Start, " ", Me.End, "}")
      End Function
      Public Property Length() As Single
         Get
            Return Me.End - Me.Start
         End Get
         Set(ByVal value As Single)
            Me.End = Me.Start + value
         End Set
      End Property

      Public Function Union(ByVal Start As Single, ByVal [end] As Single) As FloatRange
         Return New FloatRange(Math.Min(Me.Start, Math.Min(Me.End, Math.Min(Start, [end]))), Math.Max(Me.Start, Math.Max(Me.End, Math.Max(Start, [end]))))
      End Function
      Public Function Union() As FloatRange
         Return New FloatRange(Math.Min(Me.Start, Me.End), Math.Max(Me.Start, Me.End))
      End Function
      Public Function Union(ByVal other As FloatRange) As FloatRange
         Return Union(other.Start, other.End)
      End Function
      ''' <summary>meaningful suitable ome.end ordered Lists</summary>
      Public Function UnionFirstLast(ByVal list As IList(Of FloatRange)) As FloatRange
         If list.Count = 0 Then
            Return Union()
         End If
         Return Union(list(0).Start, list(list.Count - 1).End)
      End Function
      Public Function Intersect(ByVal other As FloatRange) As FloatRange
         Return New FloatRange(Math.Max(Me.Start, other.Start), Math.Min(Me.End, other.End))
      End Function
      <Obsolete("whats the use?")> _
      Public Function Shift(ByVal other As FloatRange) As FloatRange
         Return Shift(other.Start, other.Length)
      End Function
      <Obsolete("whats the use?")> _
      Public Function Shift(ByVal Start As Single, ByVal length As Single) As FloatRange
         If Me.End < Start Then
            Return Me
         End If
         If Me.Start > Start Then
            Return New FloatRange(Me.Start + length, Me.End + length)
         End If
         Return New FloatRange(Me.Start, Me.End + length)
      End Function
      Public Shared Operator =(ByVal v1 As FloatRange, ByVal v2 As FloatRange) As Boolean
         Return v1.Start = v2.Start AndAlso v1.End = v2.End
      End Operator
      Public Shared Operator <>(ByVal v1 As FloatRange, ByVal v2 As FloatRange) As Boolean
         Return Not (v1 = v2)
      End Operator
      Public Overrides Function Equals(ByVal obj As Object) As Boolean
         If Not (TypeOf obj Is FloatRange) Then
            Return False
         End If
         Return Me = CType(obj, FloatRange)
      End Function
      Public Overrides Function GetHashCode() As Integer
         Return Me.Start.GetHashCode Xor Me.End.GetHashCode
      End Function
      Public Function EachEquals(ByVal amount As Single) As Boolean
         Return Me.Start = amount AndAlso Me.End = amount
      End Function
      Public Sub SetEach(ByVal amount As Single)
         Me.Start = amount
         Me.End = amount
      End Sub
      ''' <summary>enables Linq-power to FloatRange Elements</summary>
      Public Function GetEnumerator() As IEnumerator(Of Single) Implements IEnumerable(Of Single).GetEnumerator
         Return DirectCast(New Single() {Me.Start, Me.End}, IEnumerable(Of Single)).GetEnumerator()
      End Function
      Private Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
         Return DirectCast(New Single() {Me.Start, Me.End}, IEnumerable(Of Single)).GetEnumerator()
      End Function
   End Structure
End Namespace
