﻿Imports System.Runtime.CompilerServices
Imports System.ComponentModel
Imports System.Data.Common
Imports System.IO
Imports System.Text

Namespace System.Data

   Public Module DatasetX

      <Extension()> _
      Public Function GetRankedTables(ByVal subj As DataSet) As List(Of DataTable)
         'Bevor ein Node aus dem Hashset in die Ergebnisliste verschoben wird,
         ' werden rekursiv all seine Vorgänger verschoben
         Dim tableSrc = New HashSet(Of DataTable)(subj.Tables.Cast(Of DataTable))
         GetRankedTables = New List(Of DataTable)(tableSrc.Count)
         Dim recurse As Action(Of DataTable) = Nothing
         recurse = Sub(tb)
                      If Not tableSrc.Remove(tb) Then Return 'gegen Kreisbezüge sichern
                      tb.ParentTables.ForEach(recurse)
                      GetRankedTables.Add(tb)
                   End Sub
         While tableSrc.Count > 0
            recurse(tableSrc.First)
         End While
      End Function

#Region "ReadWriteCompact"
      'Das Dataset wird durchlaufen, und alle Werte per TypConverter von und zu String umgewandelt. Schema-Validierung besteht darin, dass die Dataset-Struktur genau definiert, wann welcher TypConverter verwendet wird. Die Datei ist nur 1/7 so gross, v.a., weil sie keine Schema-Info enthält. Entscheidend aber, dass viel mehr Datentypen persistierbar sind als per Dataset.WriteXml - zB Fonts, Points und alle Enums - halt alles, wofür TypeConverter implementiert sind.


      Private Class DataRowConverter
         Private _KVPs As New List(Of KeyValuePair(Of DataColumn, AllConverter)), Table As DataTable

         Public Sub New(ByVal Table As DataTable)
            Me.Table = Table
            _KVPs = ( _
               From c In Table.Columns.Cast(Of DataColumn)() Where c.Expression = "" _
               Select NewKeyValue(c, New AllConverter(c.DataType))).ToList
         End Sub
         Public Function Parse(ByVal data As String()) As DataRow
            If data.Length <> _KVPs.Count Then Throw Me.Exception("unexpected count of input-strings")
            Dim rw = Table.NewRow
            For i = 0 To _KVPs.Count - 1
               Dim s = data(i)
               With _KVPs(i)
                  rw(.Key) = If(s = CompactReadWrite.DBNull, Convert.DBNull, .Value.FromString(s))
               End With
            Next
            Return rw
         End Function
         Public Function Format(ByVal Row As DataRow) As String()
            Dim strings(_KVPs.Count - 1) As String
            For i = 0 To _KVPs.Count - 1
               With _KVPs(i)
                  Dim val = Row(.Key)
                  strings(i) = If(Convert.IsDBNull(val), CompactReadWrite.DBNull, .Value.ToString(val))
               End With
            Next
            Return strings
         End Function
      End Class

      ''' <summary>reads from file, expecting the compact-file-format</summary>
      <Extension()> _
      Public Sub LoadCompact(ByVal Subj As DataSet, ByVal _DataFile As String)
         Using r = New CompactReadWrite(New StreamReader(_DataFile))
            For Each tb In Subj.GetRankedTables
               Dim converter = New DataRowConverter(tb)
               r.ReadTable.Select(AddressOf converter.Parse).ForEach(AddressOf tb.Rows.Add)
            Next
         End Using
      End Sub

      ''' <summary>writes to file in compact-file-format</summary>
      <Extension()> _
      Public Sub SaveCompact(ByVal Subj As DataSet, ByVal _DataFile As String)
         Using w = New CompactReadWrite(New StreamWriter(_DataFile))
            For Each tb In Subj.GetRankedTables
               w.WriteHeader(tb.TableName, _
                  From c In tb.Columns.Cast(Of DataColumn)() Where c.Expression = "" Select c.ColumnName)
               Dim converter = New DataRowConverter(tb)
               tb.All.Select(AddressOf converter.Format).ForEach(AddressOf w.WriteRow)
               w.CloseWriteTable()
            Next
         End Using
      End Sub

#End Region 'ReadWriteCompact
   End Module

End Namespace
