﻿Imports System.Runtime.CompilerServices
Imports System.Collections.Generic

Namespace System.Data

   ''' <summary>
   ''' über Datarelations im Dataset navigieren - derzeit 3 Features:
   ''' 1) GetPathTo findet die DataRelations, die von einer Tabelle zur anderen führen
   ''' 2) GetChildRelations findet alle DataRelations, die einer Tabelle untergeordnet sind
   ''' 3) IsParentOf checkt, ob eine Tabelle einer anderen übergeordnet ist
   ''' </summary>
   Public Module PathFinder

      Private _Goal As DataTable
      Private _CheckOnly As Boolean
      'da _RelationPath vom Ziel her aufgebaut wird, spart ein Stack das Umkehren der Reihenfolge
      Private _RelationPath As Stack(Of DataRelation)
      Private _Collecting As List(Of DataRelation)
      Private _KnownSteps As New Stack(Of DataTable) 'um zu vermeiden, im Kreis zu laufen
      Private _GoalFound As Boolean

      ''' <summary> wirft Exception, wenn mehr als ein Pfad gefunden </summary>
      <Extension()> _
      Public Function GetRelationPathTo(ByVal root As DataTable, ByVal goal As DataTable) As Stack(Of DataRelation)
         EnterEnumeration(root, goal, False, New Stack(Of DataRelation), Nothing)
         Return _RelationPath
      End Function

      <Extension()> _
      Public Function GetAllChildRelations(ByVal root As DataTable) As List(Of DataRelation)
         EnterEnumeration(root, Nothing, False, Nothing, New List(Of DataRelation))
         Return _Collecting
      End Function

      <Extension()> _
      Public Function IsParentOf(ByVal root As DataTable, ByVal goal As DataTable) As Boolean
         EnterEnumeration(root, goal, True, Nothing, Nothing)
         Return _GoalFound
      End Function

      Private Sub EnterEnumeration(ByVal root As DataTable, ByVal goal As DataTable, ByVal checkOnly As Boolean, ByVal relationPath As Stack(Of DataRelation), ByVal collecting As List(Of DataRelation))
         _Goal = goal
         _CheckOnly = checkOnly
         _RelationPath = relationPath
         _Collecting = collecting
         _KnownSteps.Clear()
         _GoalFound = False
         Enumerate(root)
      End Sub

      Private Sub Enumerate(ByVal Table As DataTable)
         'dieser rekursive Durchgang bedient 3 Aspekte:
         'entweder genau einen _RelationPath finden - wenns weitere gibt, Exception werfen
         'oder nur checken, ob irgendeine Verbindung existiert
         'oder alle _ChildRelations einsammeln
         If _KnownSteps.Contains(Table) Then Return 'nicht im Kreis laufen
         If Table Is _Goal Then
            If _RelationPath IsNot Nothing Then
               If _RelationPath.Count > 0 Then
                  Throw New InvalidOperationException("mehr als einen Pfad gefunden")
               End If
            End If
            _GoalFound = True
            Return
         End If
         _KnownSteps.Push(Table)
         For Each rl As DataRelation In Table.ChildRelations
            If _Collecting IsNot Nothing Then _Collecting.Add(rl)
            Enumerate(rl.ChildTable)
            If _GoalFound Then
               If _CheckOnly Then Return
               If _RelationPath IsNot Nothing Then
                  _RelationPath.Push(rl)
                  Return
               End If
            End If
         Next
         _KnownSteps.Pop()
      End Sub

   End Module


End Namespace