﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Reflection
Namespace System
   '3 classes in one file

   ''' <summary> defines the common behavior of Data&lt;T&gt;-classes </summary>
   Public MustInherit Class DataPublisher
      Public MustOverride Sub CopyTo(ByVal other As DataPublisher)
      Public MustOverride Sub OnPush()
   End Class
   ''' <summary> a wrapper for a T-Value, publishing the value by event, getter and callback </summary>
   Public Class Data2(Of T)
      Inherits DataPublisher
      Private _Value As T
      Public Event Push As Action(Of T)

      Public ReadOnly Property [Get]() As T
         Get
            Return _Value
         End Get
      End Property
      Public Sub Callback(ByVal action As Action(Of T))
         action(_Value)
      End Sub
      Public Shared Widening Operator CType(ByVal value As T) As Data2(Of T)
         Return New Data2(Of T)() With {._Value = value}
      End Operator
      Public Overrides Sub OnPush()
         RaiseEvent Push(_Value)
      End Sub
      Public Overrides Sub CopyTo(ByVal other As DataPublisher)
         DirectCast(other, Data2(Of T))._Value = _Value
      End Sub
   End Class
   ''' <summary>
   ''' derivates may publish several Data&lt;T&gt;-Fields. Void Run() inits processing of them by raising eachs Push-event
   ''' </summary>
   Public MustInherit Class ComplexData
		Public Event Start As Action 
      Public Sub New()
         ' make shure no field is null
         For Each fld In GetFieldInfos(Me.[GetType]())
            fld.SetValue(Me, Activator.CreateInstance(fld.FieldType))
         Next
      End Sub
      Private Shared Function GetFieldInfos(ByVal tp As Type) As IEnumerable(Of FieldInfo)
         Return From fld In tp.GetFields(BindingFlags.Instance Or BindingFlags.[Public]) Where fld.FieldType.BaseType.Equals(GetType(DataPublisher))
      End Function
      ''' <summary>
      ''' inits processing of the passed ComplexData-Instance by raising each Fields Push-event
      ''' </summary>
      Public Sub Run(ByVal value As ComplexData)
         Dim publishers = GetFieldInfos(Me.[GetType]()).[Select](
            Function(fld)
               Dim thisDataField = DirectCast(fld.GetValue(Me), DataPublisher)
               DirectCast(fld.GetValue(value), DataPublisher).CopyTo(thisDataField)
               Return thisDataField
            End Function).ToArray()
         RaiseEvent Start()
         publishers.ForEach(Sub(pbl) pbl.OnPush())
      End Sub
   End Class
End Namespace
